function ols_output=ols(y,x,nlag)

%---------------------------------------------------
% ols_output = ols(y,x,nlag)
% where: y = dependent variable vector    (nobs x 1)
%        x = independent variables matrix (nobs x nvar)
%        nlag = lag length for HAC SE
%---------------------------------------------------
% STRUCTURE of the OUTPUT:
%        ols_output.beta     = bhat                      -- (nvar x 1)
%        ols_output.tstat    = t-stats                   -- (nvar x 1)
%        ols_output.se       = SE of bhat                -- (nvar x 1)
%        ols_output.hac_se   = HAC SE of bhat            -- (nvar x 1)
%        ols_output.hac_stat = Robust t-stats            -- (nvar x 1)
%        ols_output.yhat     = yhat                      -- (nobs x 1)
%        ols_output.resid    = residuals                 -- (nobs x 1)
%        ols_output.sig2e    = sigma^2                   -- scalar
%        ols_output.r2       = rsquared                  -- scalar
%        ols_output.r2bar    = rbar-squared              -- scalar
%        ols_output.V        = Newey-West VarCov matrix  -- (nvar x nvar)

if (nargin ~= 3) 
    error('Wrong # of arguments to ols'); 
else
    [nobs, nvar] = size(x); 
    nobs2 = size(y);
    if (nobs ~= nobs2)
        error('x and y must have same # obs in ols'); 
    end
end

inv_xx=inv(x'*x);

% ----- beta_hat:
ols_output.beta = inv_xx*(x'*y);

% ----- y_hat:
ols_output.yhat = x*ols_output.beta;

% ----- resid:
ols_output.resid = y - ols_output.yhat;


% ----- SE(b_hat):
ols_output.sig2e = ols_output.resid'*ols_output.resid/(nobs-nvar);
tmp = ols_output.sig2e*diag(inv_xx);
ols_output.se = sqrt(tmp);
ols_output.V_ols = ols_output.sig2e*inv_xx;
% ----- t-stats:
ols_output.tstat = ols_output.beta./ols_output.se;

% ----- R^2:
rss = ols_output.resid'*ols_output.resid;
tss = (y-mean(y))'*(y-mean(y));
ols_output.r2 = 1.0 - rss/tss;

% ----- Akaike:
ols_output.aic = nobs*log(rss/nobs) + 2*nvar;

% ----- adj R^2:
rss = rss/(nobs-nvar);
tss = tss/(nobs-1.0);
ols_output.r2bar = 1 - (rss/tss); 

% ----- Newey-West SE:
emat = [];
for i=1:nvar;
emat = [emat;
        ols_output.resid'];
end;
       
hhat=emat.*x';
G=hhat*hhat';

for v=1:nlag;
    za=hhat(:,(v+1):nobs)*hhat(:,1:nobs-v)';        
    weight=(nlag+1-v)/(nlag+1);
    G=G+weight*(za+za');
end;
    
V=inv_xx*G*inv_xx;
ols_output.V=V;
 
% ----- Robust SE and t-stats:
ols_output.hac_se = sqrt(diag(V));
ols_output.hac_tstat = ols_output.beta./ols_output.hac_se;
ols_output.rsqr = ols_output.r2 ;
ols_output.rbar = ols_output.r2bar ;

